#include "ofApp.h"

//set this so that mode 2 (particle repel) goes on automatically
//make any overlap between ku and particles make noise/make ku bigger

//(both above done)

//make sound amplitude control ku's z-position (use moirave code) - he goes away if there's no sound


//use modes to grow or shrink Ku: he shrinks if he hasn't eaten in a while
//numEaten grows him if in mode 2


//--------------------------------------------------------------
void ofApp::setup(){
    
    //background graphics
    
    ofSetVerticalSync(true);
   ofBackground(25, 35, 70);
 
    ofEnableDepthTest();
    ofEnableSmoothing();
    ofEnableLighting();
       ofSetSmoothLighting(true);
    
    pointLight.enable();
    pointLight2.enable();
    pointLight3.enable();
    
    pointLight.setDiffuseColor( ofFloatColor(.79, .5, .5) );
    pointLight2.setDiffuseColor( ofFloatColor(.5, .69, .5) );
    pointLight3.setDiffuseColor( ofFloatColor(.29, .5, 0.5+0.9*scaledVol) );
    

    pointLight.setPosition(cos(ofGetElapsedTimef()), -1.01*ofGetWidth(), -10.0);
    pointLight2.setPosition(cos(ofGetElapsedTimef()), 1.05*ofGetWidth(), 10.0);
    pointLight3.setPosition(cos(ofGetElapsedTimef()), -1*ofGetWidth(), -50.0);
    
    
    float width     = ofGetWidth() * .05;
    float height    = ofGetHeight() * .12;
    
    
    //ribboncam: initialize the variable so it's off at the beginning
    usecamera = false;
        ofFill();
    
    //particles
    num = 2000;
    numEaten = 0;

    p.assign(num, demoParticle());
    currentMode = PARTICLE_MODE_REPEL;
        resetParticles();
    
    
    //soundstream
    
    
    soundStream.printDeviceList();
    
    //if you want to set a different device id
    //soundStream.setDeviceID(0); //bear in mind the device id corresponds to all audio devices, including  input-only and output-only devices.
    
    int bufferSize = 256;
    
    left.assign(bufferSize, 0.0);
    right.assign(bufferSize, 0.0);
    volHistory.assign(400, 0.0);
    
    bufferCounter	= 0;
    drawCounter		= 0;
    smoothedVol     = 0.0;
    scaledVol		= 0.0;
    
    soundStream.setup(this, 2, 2, 44100, bufferSize, 4);
    
    
    //ku
    ku.setResolution(150);
    ku.enableTextures();
    texture.load("kunew.png");
    texture.getTexture().setTextureWrap( GL_ARB_texture_swizzle, GL_ARB_texture_swizzle);
    texture.setUseTexture(true);
    texture.bind();
    // shininess is a value between 0 - 128, 128 being the most shiny //
    material.setShininess( 110 );
    // the light highlight of the material //
    material.setSpecularColor(ofColor(225, 200, 255));
    
    crossingCounter=0;
    
    //background.play();

}

//--------------------------------------------------------------
void ofApp::resetParticles(){
    
    //these are the attraction points used in the forth demo
    attractPoints.clear();
    for(int i = 0; i < 4; i++){
        attractPoints.push_back( ofPoint( ofMap(i, 0, 4, 100, ofGetWidth()-100) , ofRandom(100, ofGetHeight()), 0.0) );
    }
    
    attractPointsWithMovement = attractPoints;
    
    for(unsigned int i = 0; i < p.size(); i++){
        p[i].setMode(currentMode);
        p[i].setAttractPoints(&attractPointsWithMovement);;
        p[i].reset();
    }	
}



//--------------------------------------------------------------
void ofApp::update(){
    

    
//ribboncam:
    //don't move the points if we are using the camera
    if(!usecamera){
        ofVec3f sumOfAllPoints(0,0,0);
        for(unsigned int i = 0; i < points.size(); i++){
            points[i].z -= 4;
            sumOfAllPoints += points[i];
            if(points[i].y == kuPos.y && points[i].x == kuPos.x)
            {
                numEaten = numEaten+1;
            }

        }
        center = sumOfAllPoints / points.size();
    }
    
//particles
    
    for(unsigned int i = 0; i < p.size(); i++){
        p[i].setMode(currentMode);
        p[i].update();
        if(p[i].pos == kuPos)
        {

               numEaten = numEaten+1;
               num = num-numEaten;
            
        }
    }
    
    //add movement to points
    for(unsigned int i = 0; i < attractPointsWithMovement.size(); i++)
    {
        attractPointsWithMovement[i].x = attractPoints[i].x + ofSignedNoise(i * 10, ofGetElapsedTimef() * 0.7) * 12.0;
        attractPointsWithMovement[i].y = attractPoints[i].y + ofSignedNoise(i * -10, ofGetElapsedTimef() * 0.7) * 12.0;
        
    }
    
    //sound
    ofSoundUpdate();

    //soundstream
    //lets scale the vol up to a 0-1 range
    scaledVol = ofMap(smoothedVol, 0.0, 0.17, 0.0, 1.0, true);
    
    //lets record the volume into an array
    volHistory.push_back( scaledVol );
    
    //if we are bigger the the size we want to record - lets drop the oldest value
    if( volHistory.size() >= 400 )
    {
        volHistory.erase(volHistory.begin(), volHistory.begin()+1);
    }

    
    ku.setRadius( 77+(111*scaledVol) + float(numEaten + crossingCounter)/2);

    ku.setPosition(ofGetMouseX(), ofGetMouseY(), 40+49*scaledVol);
    
    pointLight.setSpecularColor(ofFloatColor(.99*smoothedVol, .5, .5) );
    pointLight2.setSpecularColor( ofFloatColor(.5, 0.5*cos(numEaten), .5) );
    pointLight3.setSpecularColor( ofFloatColor(.99*smoothedVol, .5, .5) );

    
}

//--------------------------------------------------------------
void ofApp::draw(){
    
    
    //---------------------------------------
    //RIBBONCAM
    //---------------------------------------
    ofPushStyle();
     //if we're using the camera, start it.
     //everything that you draw between begin()/end() shows up from the view of the camera
     if(usecamera){
     camera.begin();
     }
    
    ofSetColor(40);
   
     //do the same thing from the first example...
     ofMesh mesh;
     mesh.setMode(OF_PRIMITIVE_TRIANGLE_STRIP);
    
     for(unsigned int i = 1; i < points.size(); i++){
     
     //find this point and the next point
     ofVec3f thisPoint = points[i-1];
     ofVec3f nextPoint = points[i];
     
     //get the direction from one to the next.
     //the ribbon should fan out from this direction
     ofVec3f direction = (nextPoint - thisPoint);
     
     //get the distance from one point to the next
     float distance = direction.length();
     
     //get the normalized direction. normalized vectors always have a length of one
     //and are really useful for representing directions as opposed to something with length
     ofVec3f unitDirection = direction.getNormalized();
     
     //find both directions to the left and to the right
     ofVec3f toTheLeft = unitDirection.getRotated(-90, ofVec3f(0,0,1));
     ofVec3f toTheRight = unitDirection.getRotated(90, ofVec3f(0,0,1));
     
     //use the map function to determine the distance.
     //the longer the distance, the narrower the line.
     //this makes it look a bit like brush strokes
     float thickness = ofMap(distance, 0, 60, 20, 2, true);
     
     //calculate the points to the left and to the right
     //by extending the current point in the direction of left/right by the length
     ofVec3f leftPoint = thisPoint+toTheLeft*thickness;
     ofVec3f rightPoint = thisPoint+toTheRight*thickness;
     
     //add these points to the triangle strip
     mesh.addVertex(ofVec3f(leftPoint.x, leftPoint.y, leftPoint.z));
     mesh.addVertex(ofVec3f(rightPoint.x, rightPoint.y, rightPoint.z));
     }
     
     //end the shape
     mesh.draw();
    
     //if we're using the camera, take it away
     if(usecamera){
    	camera.end();
     }
    ofPopStyle();
    //---------------------------------------
    //particles
    //---------------------------------------

    
    
    ofPushStyle();
    for(unsigned int i = 0; i < p.size(); i++)
    {
        p[i].draw();
    }
    

    if( currentMode == PARTICLE_MODE_REPEL ){
        for(unsigned int i = 0; i < attractPoints.size(); i++){
            ofNoFill();
            ofDrawCircle(attractPointsWithMovement[i], 10);
            ofFill();
            ofDrawCircle(attractPointsWithMovement[i], 4);
         
            if (attractPointsWithMovement[i].x == ofGetMouseX() && attractPointsWithMovement[i].y == ofGetMouseY())
                {
                numEaten = numEaten+1;
                num = num-numEaten;
                
                string numString = to_string(num);
                string numEatenString = to_string(numEaten);
                ofPushStyle();
                    ofSetColor(200);
                    ofDrawBitmapString(numEatenString + " soundbites eaten...", 20., 50.);
                    ofDrawBitmapString(numString + " soundbites left!", 20., 70.);
                ofPopStyle();
                }

        }
    }
    
    ofPopStyle();

    //---------------------------------------
    //ku
    //---------------------------------------
    
    ofPoint kuPos(ofGetMouseX(), ofGetMouseY(), 40+49*scaledVol);

    
    ku.setMode( OF_PRIMITIVE_TRIANGLES );
    triangles = ku.getMesh().getUniqueFaces();
    ku.getMesh().setFromTriangles( triangles, true );
    ku.mapTexCoordsFromTexture( texture.getTexture() );
    texture.getTexture().bind();
    material.begin();
   
    ku.draw();

    material.end();
    
    float spinX = sin(ofGetElapsedTimef()*.35f);
    float spinY = cos(ofGetElapsedTimef()*.075f);
    

    ku.rotate(spinX, 1.0, 0.0, 0.0);
    ku.rotate(spinY, 0, 1.0, 0.0);


    ofSetColor(pointLight.getDiffuseColor());
    pointLight.draw();
    ofSetColor(pointLight2.getDiffuseColor());
    pointLight2.draw();
    ofSetColor(pointLight3.getDiffuseColor());
    pointLight3.draw();
    //vidGrabber.getTexture().bind();
}

//--------------------------------------------------------------
void ofApp::audioIn(float * input, int bufferSize, int nChannels){
    
    float curVol = 0.0;
    // samples are "interleaved"
    int numCounted = 0;
    
    //lets go through each sample and calculate the root mean square which is a rough way to calculate volume
    for (int i = 0; i < bufferSize; i++){
        left[i]		= input[i*2]*0.5;
        right[i]	= input[i*2+1]*0.5;
        
        curVol += left[i] * left[i];
        curVol += right[i] * right[i];
        numCounted+=2;
        
    }
    
    //this is how we get the mean of rms :)
    curVol /= (float)numCounted;
    
    // this is how we get the root of rms :)
    curVol = sqrt( curVol );
    
    smoothedVol *= 0.93;
    smoothedVol += 0.07 * curVol;

    bufferCounter++;
    
    
    
}


//--------------------------------------------------------------
void ofApp::keyPressed(int key){

    
    //RIBBONCAM
    //hitting c key swaps the camera view
    if( key == 'c'){
        
    usecamera = !usecamera;
    }

    //particles
    if( key == '1'){
        currentMode = PARTICLE_MODE_ATTRACT;
        currentModeStr = "1 - PARTICLE_MODE_ATTRACT: attracts to mouse";
    }
    if( key == '2'){
        currentMode = PARTICLE_MODE_REPEL;
        currentModeStr = "2 - PARTICLE_MODE_REPEL: repels from mouse";
        resetParticles();
    }
    if( key == '3'){
        currentMode = PARTICLE_MODE_NEAREST_POINTS;
        currentModeStr = "3 - PARTICLE_MODE_NEAREST_POINTS: hold 'f' to disable force";
      //  usecamera = !usecamera;
    }
    if( key == '4'){
        currentMode = PARTICLE_MODE_NOISE;
        currentModeStr = "4 - PARTICLE_MODE_NOISE: snow particle simulation";
        resetParticles();
    }	
    
    if( key == ' ' ){
        resetParticles();
    }


}

//--------------------------------------------------------------
void ofApp::keyReleased(int key){

}

//--------------------------------------------------------------
void ofApp::mouseMoved(int x, int y ){
    

    //RIBBONCAM: 
    //if we are using the camera, the mouse moving should rotate it around the whole sculpture
    if(usecamera){
        float rotateAmount = ofMap(ofGetMouseX(), 0, ofGetWidth(), 0, 360);
        ofVec3f furthestPoint;
        if (points.size() > 0) {
            furthestPoint = points[0];
        }
        else
        {
            furthestPoint = ofVec3f(x, y, 0);
        }
        
        ofVec3f directionToFurthestPoint = (furthestPoint - center);
        ofVec3f directionToFurthestPointRotated = directionToFurthestPoint.getRotated(rotateAmount, ofVec3f(0,1,0));
        camera.setPosition(center + directionToFurthestPointRotated);
        camera.lookAt(center);
    }
    //otherwise add points like before
    else{
        ofVec3f mousePoint(x,y,0);
        points.push_back(mousePoint);
    }
    
    
    
}

//--------------------------------------------------------------
void ofApp::mouseDragged(int x, int y, int button){
    
   

}

//--------------------------------------------------------------
void ofApp::mousePressed(int x, int y, int button){
        bMousePressed = true;

}

//--------------------------------------------------------------
void ofApp::mouseReleased(int x, int y, int button){

}

//--------------------------------------------------------------
void ofApp::mouseEntered(int x, int y){

}

//--------------------------------------------------------------
void ofApp::mouseExited(int x, int y){

}

//--------------------------------------------------------------
void ofApp::windowResized(int w, int h){

}

//--------------------------------------------------------------
void ofApp::gotMessage(ofMessage msg){

}

//--------------------------------------------------------------
void ofApp::dragEvent(ofDragInfo dragInfo){ 

}
